{-# LANGUAGE DataKinds #-}
{-# LANGUAGE DeriveAnyClass #-}
{-# LANGUAGE DuplicateRecordFields #-}
{-# LANGUAGE OverloadedRecordDot #-}

module Jon.Server where

import Control.Monad.IO.Class (liftIO)
import Data.Aeson (FromJSON, ToJSON)
import Data.Text (Text)
import Data.Int (Int32)
import Database.PostgreSQL.Simple (Connection)
import GHC.Generics (Generic)
import Servant

import Jon.Garfield.Types

import qualified Jon.Garfield.Queries as Queries

-- API and types

type JonAPI = "getUnsoundBarcodes" :> ReqBody '[JSON] GetUnsoundBarcodesP :> Post '[JSON] GetUnsoundBarcodesR

data GetUnsoundBarcodesP = GetUnsoundBarcodesP
    { location :: Int32
    } deriving (Show, Generic, FromJSON)

data GetUnsoundBarcodesR = GetUnsoundBarcodesR
    { unsoundBarcodes :: [UnsoundBarcode]
    } deriving (Show, Generic, ToJSON)

data UnsoundBarcode = UnsoundBarcode
    { barcode :: Text
    , name :: Text
    , entries :: Int
    , unitsLeft :: Int
    } deriving (Show, Generic, ToJSON)

-- server

server :: Connection -> Server JonAPI
server conn = getUnsoundBarcodes
    where
        getUnsoundBarcodes :: GetUnsoundBarcodesP -> Handler GetUnsoundBarcodesR
        getUnsoundBarcodes params = do
            rows <- liftIO $ Queries.runSelect conn $ Queries.unsoundBarcodes $ mkLocationId params.location
            pure $ GetUnsoundBarcodesR $ map mkUnsoundBarcode rows
            where
                mkUnsoundBarcode (a, b, c, d) = UnsoundBarcode a b (fromIntegral c) (fromIntegral d)
